package com.bureau.devicefingerprint

import android.app.ActivityManager
import android.app.KeyguardManager
import android.app.admin.DevicePolicyManager
import android.content.Context
import android.content.pm.Signature
import android.hardware.SensorManager
import android.hardware.input.InputManager
import android.media.MediaCodecList
import android.media.RingtoneManager
import androidx.core.hardware.fingerprint.FingerprintManagerCompat
import com.bureau.base.extensions.getFromClipBoard
import com.bureau.devicefingerprint.datacollectors.AdvertisingProvider
import com.bureau.devicefingerprint.datacollectors.AndroidIdProvider
import com.bureau.devicefingerprint.datacollectors.AppSignaturesProvider
import com.bureau.devicefingerprint.datacollectors.BatteryInfoProvider
import com.bureau.devicefingerprint.datacollectors.BatteryInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.CallInfoProvider
import com.bureau.devicefingerprint.datacollectors.CameraInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.CodecInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.CpuInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.DebuggableInfoProvider
import com.bureau.devicefingerprint.datacollectors.DevicePersonalizationInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.DeviceSecurityInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.DeviceStorageProvider
import com.bureau.devicefingerprint.datacollectors.EmulatorPropertiesProvider
import com.bureau.devicefingerprint.datacollectors.FactoryResetTimeProvider
import com.bureau.devicefingerprint.datacollectors.FileProvisionsProvider
import com.bureau.devicefingerprint.datacollectors.FingerprintSensorInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.GpuInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.GsfIdProvider
import com.bureau.devicefingerprint.datacollectors.HookDetectionCheckProvider
import com.bureau.devicefingerprint.datacollectors.InputDevicesDataSourceImpl
import com.bureau.devicefingerprint.datacollectors.InstallInfoProvider
import com.bureau.devicefingerprint.datacollectors.KeyStoreProvider
import com.bureau.devicefingerprint.datacollectors.LocationInfoProvider
import com.bureau.devicefingerprint.datacollectors.MediaDrmIdProvider
import com.bureau.devicefingerprint.datacollectors.MemInfoProvider
import com.bureau.devicefingerprint.datacollectors.NetworkInfoProvider
import com.bureau.devicefingerprint.datacollectors.NewRawParametersProvider
import com.bureau.devicefingerprint.datacollectors.NewRawParametersProviderImpl
import com.bureau.devicefingerprint.datacollectors.OsBuildInfoProvider
import com.bureau.devicefingerprint.datacollectors.OsBuildInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.PackageManagerDataSourceImpl
import com.bureau.devicefingerprint.datacollectors.ScreenInfoProvider
import com.bureau.devicefingerprint.datacollectors.SensorDataSourceImpl
import com.bureau.devicefingerprint.datacollectors.SettingsDataSourceImpl
import com.bureau.devicefingerprint.datacollectors.SocialEngineeringInfoProvider
import com.bureau.devicefingerprint.datacollectors.SocialEngineeringInfoProviderImpl
import com.bureau.devicefingerprint.datacollectors.SystemPropertiesProvider
import com.bureau.devicefingerprint.datacollectors.TelecomBaseStationInfoProvider
import com.bureau.devicefingerprint.datacollectors.WebInfoProvider
import com.bureau.devicefingerprint.datacollectors.WifiInfoProvider
import com.bureau.devicefingerprint.datacollectors.WifiInfoProviderImpl
import com.bureau.devicefingerprint.models.CredentialBuilder
import com.bureau.devicefingerprint.models.devicedataholder.AndroidId
import com.bureau.devicefingerprint.models.devicedataholder.BatteryInfo
import com.bureau.devicefingerprint.models.devicedataholder.CameraCoreInfo
import com.bureau.devicefingerprint.models.devicedataholder.CameraInfo
import com.bureau.devicefingerprint.models.devicedataholder.CodecInfo
import com.bureau.devicefingerprint.models.devicedataholder.CpuInfo
import com.bureau.devicefingerprint.models.devicedataholder.CurrentLocation
import com.bureau.devicefingerprint.models.devicedataholder.DebuggerInfo
import com.bureau.devicefingerprint.models.devicedataholder.DeviceDataHolder
import com.bureau.devicefingerprint.models.devicedataholder.DevicePersonalizationInfo
import com.bureau.devicefingerprint.models.devicedataholder.DeviceSecurityInfo
import com.bureau.devicefingerprint.models.devicedataholder.EmulatorInfo
import com.bureau.devicefingerprint.models.devicedataholder.FingerPrintSensorInfo
import com.bureau.devicefingerprint.models.devicedataholder.GpuInfo
import com.bureau.devicefingerprint.models.devicedataholder.GsfId
import com.bureau.devicefingerprint.models.devicedataholder.InputDeviceData
import com.bureau.devicefingerprint.models.devicedataholder.InputDevicesInfo
import com.bureau.devicefingerprint.models.devicedataholder.LastKnownLocation
import com.bureau.devicefingerprint.models.devicedataholder.LocationInfo
import com.bureau.devicefingerprint.models.devicedataholder.MediaDrmId
import com.bureau.devicefingerprint.models.devicedataholder.MemInfo
import com.bureau.devicefingerprint.models.devicedataholder.MetaData
import com.bureau.devicefingerprint.models.devicedataholder.NdkInfo
import com.bureau.devicefingerprint.models.devicedataholder.NetworkInfo
import com.bureau.devicefingerprint.models.devicedataholder.OsBuildInfo
import com.bureau.devicefingerprint.models.devicedataholder.PackageManagerInfo
import com.bureau.devicefingerprint.models.devicedataholder.PastRealTimeDatHolder
import com.bureau.devicefingerprint.models.devicedataholder.RawParameters
import com.bureau.devicefingerprint.models.devicedataholder.ScreenInfoHolder
import com.bureau.devicefingerprint.models.devicedataholder.SensorData
import com.bureau.devicefingerprint.models.devicedataholder.SensorsInfo
import com.bureau.devicefingerprint.models.devicedataholder.SettingsInfo
import com.bureau.devicefingerprint.models.devicedataholder.SignaturesInfo
import com.bureau.devicefingerprint.models.devicedataholder.SocialEngineeringInfo
import com.bureau.devicefingerprint.models.devicedataholder.SystemApplication
import com.bureau.devicefingerprint.models.devicedataholder.SystemProperties
import com.bureau.devicefingerprint.models.devicedataholder.UserApplication
import com.bureau.devicefingerprint.models.devicedataholder.WebInfo
import com.bureau.devicefingerprint.models.devicedataholder.WifiInfoParameters
import com.bureau.devicefingerprint.tools.DependencyHelpers
import com.bureau.devicefingerprint.tools.NDKMappers
import com.bureau.devicefingerprint.tools.PRISM_SDK_VERSION
import com.bureau.devicefingerprint.tools.SharedPreferenceHelper
import com.bureau.devicefingerprint.tools.executeSafe
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.io.UnsupportedEncodingException
import java.nio.charset.Charset

class DataMapper(var context: Context) {

    private var deviceDataHolder = DeviceDataHolder()
    private var ndkMappers: NDKMappers = NDKMappers()
    private var hasLocationPermission = false
    private var vendorName = ""
    private var provider: LocationInfoProvider? = null

    fun getAsyncOSData(
        credentialBuilder: CredentialBuilder?,
        deviceDataHolder: DeviceDataHolder
    ): DeviceDataHolder {
        getNDKData()

        deviceDataHolder.androidId_ =
            AndroidId(AndroidIdProvider(context.contentResolver).getAndroidId())

        val batteryInfoProvider: BatteryInfoProvider = BatteryInfoProviderImpl(context)
        deviceDataHolder.batteryInfo_ = BatteryInfo(
            batteryCapacity_ = batteryInfoProvider.batteryTotalCapacity(),
            batteryHealth_ = batteryInfoProvider.batteryHealth(),
            isCharging_ = batteryInfoProvider.isCharging()
        )

        deviceDataHolder.cameraInfo_ = getCameraInfoData()
        deviceDataHolder.elapsedTime_ = OsBuildInfoProviderImpl().getElapsedTime()
        deviceDataHolder.encryptionStatus_ = DeviceStorageProvider(context).getEncryptionStatus()
        deviceDataHolder.cpuInfo_ = getCpuInfoData()
        deviceDataHolder.devicePersonalizationInfo_ = getDevicePersonalizationInfo()
        deviceDataHolder.deviceSecurityInfo_ = getDeviceSecurityInfo()
        deviceDataHolder.fingerPrintSensorInfo_ = FingerPrintSensorInfo(
            sensorStatus_ = FingerprintSensorInfoProviderImpl(
                FingerprintManagerCompat.from(context)
            ).getStatus().name
        )
        deviceDataHolder.gpuInfo_ =
            GpuInfo(glesVersion_ = GpuInfoProviderImpl(context.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager).glesVersion())
        deviceDataHolder.gsfId_ =
            GsfId(gsfId_ = GsfIdProvider(context.contentResolver).getGsfAndroidId())
        deviceDataHolder.inputDevicesInfo_ = getInputDevicesInfo()

        val memInfoProvider = DependencyHelpers.createMemoryInfoProvider(context)
        deviceDataHolder.memInfo_ = getMemInfoData(memInfoProvider)

        val osBuildInfoProvider: OsBuildInfoProvider = OsBuildInfoProviderImpl()
        deviceDataHolder.osBuildInfo_ = getOSBuildData(osBuildInfoProvider)
        deviceDataHolder.packageManagerInfo_ = getPackageManagerInfoData()
        deviceDataHolder.sensorsInfo_ = getSensorInfoData()
        deviceDataHolder.settingsInfo_ = getSettingsInfoData()
        deviceDataHolder.emulatorInfo_ = EmulatorInfo(
            emulatorIp_ = EmulatorPropertiesProvider(context).getEmulatorIP(),
            qEMUVendor_ = vendorName
        )
        deviceDataHolder.systemProperties_ = getSystemProperties()
        deviceDataHolder.networkInfo_ = NetworkInfo(
            iPV4_ = NetworkInfoProvider(context).getIpv4Address(),
            iPV6_ = NetworkInfoProvider(context).getIpv6Address(),
            ethernetMACAddress_ = NetworkInfoProvider(context).getEthernetMACAddress(),
            wifiMacAddress_ = NetworkInfoProvider(context).getWifiMACAddress(),
            isVpnConnected_ = NetworkInfoProvider(context).isConnectedToVPN(),
            isWifiConnected_ = NetworkInfoProvider(context).getIsWifiConnected(),
            isCellularConnected_ = NetworkInfoProvider(context).getIsCellularConnected()
        )
        deviceDataHolder.webInfo_ = WebInfo(userAgent_ = WebInfoProvider().getUserAgent())
        deviceDataHolder.keyStoreKey_ = KeyStoreProvider(context).getKey()
        deviceDataHolder.userId_ = credentialBuilder?.userId ?: ""
        deviceDataHolder.flow_ = credentialBuilder?.flow ?: ""
        deviceDataHolder.sdkVersion_ = PRISM_SDK_VERSION
        deviceDataHolder.debuggerInfo_ = DebuggerInfo(
            isDebuggerAttached_ = DebuggableInfoProvider(context).isDebuggerAttached(),
            isRooted_ = DebuggableInfoProvider(context).isRooted()
        )
        deviceDataHolder.signaturesInfo_ = getSignaturesInfo()
        deviceDataHolder.sessionId_ = credentialBuilder?.sessionId ?: ""
        deviceDataHolder.voiceCallMode_ = CallInfoProvider(context).audioMode()
        deviceDataHolder.appDirectorypath_ = context.filesDir.absolutePath
        deviceDataHolder.screenInfo_ = ScreenInfoHolder(
            screenCount_ = ScreenInfoProvider(context).getScreenCount(),
            drawableDensity_ = ScreenInfoProvider(context).getDrawableDensity(),
            layoutSize_ = ScreenInfoProvider(context).getLayoutSize(),
            layoutHeight_ = ScreenInfoProvider(context).getLayoutHeight(),
            layoutWidth_ = ScreenInfoProvider(context).getLayoutWidth()
        )
        deviceDataHolder.lastfactoryResetTime_ = FactoryResetTimeProvider.getFactoryResetTime(context.packageManager, context)
        deviceDataHolder.telecomBaseStationInfo_ = TelecomBaseStationInfoProvider(context).getTelecomBaseStationInfo()
        val newRawParametersProvider: NewRawParametersProvider = NewRawParametersProviderImpl(context)
        deviceDataHolder.rawParameters = RawParameters(
            isMuted = newRawParametersProvider.isMuted(),
            volumeLevel = newRawParametersProvider.getVolumePercentage(),
            batteryLevel = batteryInfoProvider.getBatteryLevel(),
            batteryTemperatureInCentigrade = batteryInfoProvider.getBatterTemperature(),
            batteryVoltage = batteryInfoProvider.getBatteryVoltage(),
            buildId = osBuildInfoProvider.getBuildId(),
            buildTime = osBuildInfoProvider.getBuildTime(),
            freeExternalStorageSpace = memInfoProvider.getFreeExternalStorageSpace(),
            freeInternalStorageSpace = memInfoProvider.getFreeInternalStorageSpace(),
            pasteBoardString = context.getFromClipBoard() ?: "",
            brightness = newRawParametersProvider.getBrightness()
        )

        val socialEngineeringInfoProvider: SocialEngineeringInfoProvider = SocialEngineeringInfoProviderImpl(context)
        deviceDataHolder.socialEngineeringInfo = SocialEngineeringInfo(isSocialEngineeringAppInstalled = socialEngineeringInfoProvider.getIsSocialEngineeringAppInstalled())

        val wifiInfoProvider: WifiInfoProvider = WifiInfoProviderImpl(context)
        deviceDataHolder.wifiInfoParameters = WifiInfoParameters(wifiInfoProvider.getSSID(), wifiInfoProvider.getBSSID(), wifiInfoProvider.getRSSI())

        return deviceDataHolder
    }

    fun getMediaDrmId(): MediaDrmId {
        return MediaDrmId(drmId_ = MediaDrmIdProvider().getMediaDrmId())
    }

    fun getMediaCodecInfo(): CodecInfo {
        return getCodecInfoData(MediaCodecList(MediaCodecList.ALL_CODECS))
    }

    fun getIsHookingDetected(): Boolean {
        return HookDetectionCheckProvider(context).isHookingDetected()
    }

    fun addGeocodeData(deviceData: DeviceDataHolder): DeviceDataHolder {
        if (provider?.decodeCityName(provider?.currentLocation, context).isNullOrEmpty()) {
            deviceData.locationInfo_?.currentLocation_?.city_ = provider?.decodeCityName(provider?.lastKnownLocation, context)
        } else {
            deviceData.locationInfo_?.currentLocation_?.city_ = provider?.decodeCityName(provider?.currentLocation, context)
        }

        deviceData.locationInfo_?.lastKnownLocation_?.city_ = provider?.decodeCityName(provider?.lastKnownLocation, context)
        if (provider?.decodeCountryName(provider?.currentLocation, context).isNullOrEmpty()) {
            deviceData.locationInfo_?.currentLocation_?.country_ = provider?.decodeCountryName(provider?.lastKnownLocation, context)
        } else {
            deviceData.locationInfo_?.currentLocation_?.country_ = provider?.decodeCountryName(provider?.currentLocation, context)
        }

        deviceData.locationInfo_?.lastKnownLocation_?.country_ = provider?.decodeCountryName(provider?.lastKnownLocation, context)
        if (provider?.decodeRegionName(provider?.currentLocation, context).isNullOrEmpty()) {
            deviceData.locationInfo_?.currentLocation_?.region_ = provider?.decodeRegionName(provider?.lastKnownLocation, context)
        } else {
            deviceData.locationInfo_?.currentLocation_?.region_ = provider?.decodeRegionName(provider?.currentLocation, context)
        }

        deviceData.locationInfo_?.lastKnownLocation_?.region_ = provider?.decodeRegionName(provider?.lastKnownLocation, context)
        return deviceData
    }

    fun getSynchronisedOSData(
        hasLocationPermission: Boolean
    ): DeviceDataHolder {
        this.hasLocationPermission = hasLocationPermission
        deviceDataHolder.locationInfo_ = getLocationInfo()

        val (gpsEnabled, networkEnabled) = provider?.getAreGpsAndNetworkEnabled() ?: Pair(false, false)
        deviceDataHolder.metaData = MetaData(gpsEnabled, networkEnabled)

        deviceDataHolder.uniqueFileId_ = FileProvisionsProvider(context).getUniqueIDFromFile()
        return deviceDataHolder
    }

    suspend fun getAsynchronousIds(deviceData: DeviceDataHolder): DeviceDataHolder = withContext(Dispatchers.IO) {
        deviceData.advertisingID_ = AdvertisingProvider(context).getAdvertisingId()
        deviceData
    }

    private fun getSignaturesInfo(): SignaturesInfo {
        val list: ArrayList<String> = ArrayList()
        val signatures: Array<Signature>? = AppSignaturesProvider(context).getSignatures()

        signatures?.forEach {
            list.add(it.toCharsString())
        }
        return SignaturesInfo(
            isPlayStoreInstall_ = InstallInfoProvider(context).getIsPlayStoreInstall(),
            signatureHashcode_ = list
        )
    }

    private fun getNDKData() {
        deviceDataHolder.ndkInfo_ =
            NdkInfo(
                applicationId_ = executeSafe({ ndkMappers.getApplicationId() }, ""),
                installPath_ = executeSafe({ ndkMappers.getSourceDir(context) }, "")
            )
        val byteArray: ByteArray = ndkMappers.getEmulatorVendor()
        try {
            val nativeString = String(byteArray, Charset.defaultCharset())
            vendorName = nativeString
        } catch (e: UnsupportedEncodingException) {
            BureauAPI.handleException(e)
            e.printStackTrace()
        }
    }

    private fun getCameraInfoData(): CameraInfo {
        val list: ArrayList<CameraCoreInfo> = ArrayList()
        CameraInfoProviderImpl().getCameraInfo().forEach {
            list.add(
                CameraCoreInfo(
                    cameraName_ = it.cameraName,
                    cameraOrientation_ = it.cameraOrientation,
                    cameraType_ = it.cameraType
                )
            )
        }
        return CameraInfo(cameraCoreInfo_ = list)
    }

    private fun getCodecInfoData(codecList: MediaCodecList): CodecInfo {
        val codecInfo = CodecInfo()

        CodecInfoProviderImpl(codecList).codecsList().forEach {
            codecInfo.name_ = it.name
            val list: ArrayList<String> = ArrayList()
            it.capabilities.forEach { it2 ->
                list.add(it2)
            }
            codecInfo.capabilities_ = list
        }
        return codecInfo
    }

    private fun getCpuInfoData(): CpuInfo {
        val map: HashMap<String, String> = HashMap()
        CpuInfoProviderImpl().cpuInfo().asIterable().forEach {
            map[it.key] = it.value
        }

        return CpuInfo(
            abiType_ = CpuInfoProviderImpl().abiType(),
            coreCount_ = CpuInfoProviderImpl().coresCount(),
            map
        )
    }

    private fun getDevicePersonalizationInfo(): DevicePersonalizationInfo {
        val list: ArrayList<String> = ArrayList()

        val provider = DevicePersonalizationInfoProviderImpl(
            RingtoneManager(context),
            context.assets,
            context.resources.configuration
        )
        provider.availableLocales().forEach {
            list.add(it)
        }
        return DevicePersonalizationInfo(
            defaultLanguage_ = provider.defaultLanguage(),
            regionOrCountry_ = provider.regionCountry(),
            ringtoneSource_ = provider.ringtoneSource(),
            timeZone_ = provider.timezone(),
            availableLocales_ = list
        )
    }

    private fun getDeviceSecurityInfo(): DeviceSecurityInfo {
        val map: HashMap<String, String> = HashMap()

        val provider = DeviceSecurityInfoProviderImpl(
            context.getSystemService(Context.DEVICE_POLICY_SERVICE) as DevicePolicyManager,
            context.getSystemService(Context.KEYGUARD_SERVICE) as KeyguardManager
        )
        provider.securityProvidersToMap().asIterable().forEach {
            map[it.key] = it.value
        }

        return DeviceSecurityInfo(
            encryptionStatus_ = provider.encryptionStatus(),
            isPinEnabled_ = provider.isPinSecurityEnabled(),
            securityProviders_ = map
        )
    }

    private fun getInputDevicesInfo(): InputDevicesInfo {
        val list: ArrayList<InputDeviceData> = ArrayList()
        val provider =
            InputDevicesDataSourceImpl(context.getSystemService(Context.INPUT_SERVICE) as InputManager)
        provider.getInputDeviceData().forEach {
            list.add(InputDeviceData(name_ = it.name, vendor_ = it.vendor))
        }
        return InputDevicesInfo(inputDeviceData_ = list)
    }

    private fun getMemInfoData(memInfoProvider: MemInfoProvider): MemInfo {
        return MemInfo(
            totalRAM_ = memInfoProvider.totalRAM(),
            totalExternalStorageSpace_ = memInfoProvider.totalExternalStorageSpace(),
            totalInternalStorageSpace_ = memInfoProvider.totalInternalStorageSpace()
        )
    }

    private fun getOSBuildData(provider: OsBuildInfoProvider): OsBuildInfo {
        return OsBuildInfo(
            androidVersion_ = provider.androidVersion(),
            fingerprint_ = provider.fingerprint(),
            kernelVersion_ = provider.kernelVersion(),
            manufacturerName_ = provider.manufacturerName(),
            modelName_ = provider.modelName(),
            sdkVersion_ = provider.sdkVersion(),
            device_ = provider.device(),
            product_ = provider.product(),
            host_ = provider.host(),
            brand_ = provider.brand(),
            board_ = provider.board()
        )
    }

    private fun getPackageManagerInfoData(): PackageManagerInfo {
        val userList: ArrayList<UserApplication> = ArrayList()
        val systemList: ArrayList<SystemApplication> = ArrayList()

        val provider = PackageManagerDataSourceImpl(context.packageManager, context)

        /*provider.getSystemApplicationsList().forEach {
            systemList.add(SystemApplication(it.packageName))
        }
        provider.getApplicationsList().forEach {
            userList.add(UserApplication(it.packageName))
        }*/

        return PackageManagerInfo(
            packageVersion_ = provider.getPackageVersion(),
            isPackageDebuggable_ = DebuggableInfoProvider(context).isDebuggable(),
            systemApplicationList_ = systemList,
            userApplicationList_ = userList
        )
    }

    private fun getSensorInfoData(): SensorsInfo {
        val list: ArrayList<SensorData> = ArrayList()

        val provider =
            SensorDataSourceImpl(context.getSystemService(Context.SENSOR_SERVICE) as SensorManager)

        provider.sensors().forEach {
            list.add(SensorData(sensorName_ = it.sensorName, sensorVendorName_ = it.vendorName))
        }

        return SensorsInfo(sensorData_ = list)
    }

    private fun getSettingsInfoData(): SettingsInfo {
        val provider = SettingsDataSourceImpl(context.contentResolver)

        return SettingsInfo(
            accessibilityEnabled_ = provider.accessibilityEnabled(),
            adbEnabled_ = provider.adbEnabled(),
            alarmAlertPath_ = provider.alarmAlertPath(),
            dataRoamingEnabled_ = provider.dataRoamingEnabled(),
            dateFormat_ = provider.dateFormat(),
            defaultInputMethod_ = provider.defaultInputMethod(),
            developmentSettingsEnabled_ = provider.developmentSettingsEnabled(),
            endButtonBehaviour_ = provider.endButtonBehaviour(),
            fontScale_ = provider.fontScale(),
            httpProxy_ = provider.httpProxy(),
            rttCallingMode_ = provider.rttCallingMode(),
            screenOffTimeout_ = provider.screenOffTimeout(),
            textAutoPunctuate_ = provider.textAutoPunctuate(),
            textAutoReplaceEnable_ = provider.textAutoReplaceEnable(),
            touchExplorationEnabled_ = provider.touchExplorationEnabled(),
            transitionAnimationScale_ = provider.transitionAnimationScale(),
            windowAnimationScale_ = provider.windowAnimationScale()
        )
    }

    private fun getSystemProperties(): SystemProperties {
        val provider = SystemPropertiesProvider()

        return SystemProperties(
            attributeData_ = provider.getAttributeData(),
            bootMode_ = provider.getBootMode(),
            density_ = provider.getLCDDensity(),
            deviceProps_ = provider.getProps(),
            fakeCamera_ = provider.getFakeCamera(),
            kernelGlESEnabled_ = provider.getGLES(),
            kernelVMRunning_ = provider.getKernelData(),
            mainKeyEnabled_ = provider.getMainKey(),
            serialNo_ = provider.getSerialNo(),
            kernelQEMUEnabled_ = provider.getQEMU(),
            systemProperties = provider.getSystemProperties()
        )
    }

    private fun getLocationInfo(): LocationInfo {
        val provider = LocationInfoProvider(context)
        this.provider = provider
        provider.getKnownLocation()

        return LocationInfo(
            currentLocation_ = CurrentLocation(
                latitude_ = LocationInfoProvider.locationLatitude.ifEmpty { provider.getLastKnownLocation().latitude },
                longitude_ = LocationInfoProvider.locationLongitude.ifEmpty { provider.getLastKnownLocation().longitude },
                gpsPrecision_ = if (LocationInfoProvider.precision.equals(0f)) provider.getLastKnownLocation().locationPrecision else LocationInfoProvider.precision
            ),
            lastKnownLocation_ = LastKnownLocation(
                latitude_ = provider.getLastKnownLocation().latitude,
                longitude_ = provider.getLastKnownLocation().longitude,
                gpsPrecision_ = if (LocationInfoProvider.precision.equals(0f)) provider.getLastKnownLocation().locationPrecision else LocationInfoProvider.precision
            ),
            isLocationEnabled_ = provider.getIsLocationEnabled(),
            isLocationPermissionEnabled_ = hasLocationPermission,
            isMockGPS_ = provider.getLastKnownLocation().isMockLocation
        ).also {
            provider.removeListener()
        }
    }

    fun addPastRealData(deviceDataHolder: DeviceDataHolder): DeviceDataHolder {
        deviceDataHolder.pastRealTimeDataInfo_ = PastRealTimeDatHolder(
            pastVpnStatus_ = SharedPreferenceHelper.getSharedPreferenceBoolean(
                context,
                SharedPreferenceHelper.isPastVPNData,
                defValue = false,
                isDataMonitoring = true
            ),
            pastMockGPSStatus_ = SharedPreferenceHelper.getSharedPreferenceBoolean(
                context,
                SharedPreferenceHelper.isPastMockGPSData,
                defValue = false,
                isDataMonitoring = true
            ),
            pastDebuggerStatus_ = SharedPreferenceHelper.getSharedPreferenceBoolean(
                context,
                SharedPreferenceHelper.isPastDebuggerData,
                defValue = false,
                isDataMonitoring = true
            )
        )
        return deviceDataHolder
    }
}
