package com.bureau.devicefingerprint.datacollectors

import android.content.Context
import android.content.pm.ApplicationInfo
import android.content.pm.PackageManager
import com.bureau.devicefingerprint.tools.executeSafe

object FactoryResetTimeProvider {
    fun getFactoryResetTime(packageManager: PackageManager, context: Context): Long {
        return executeSafe({
            val applicationInfos = packageManager
                .getInstalledApplications(PackageManager.GET_META_DATA)
                .filter {
                    !isSystemPackage(it) && getIsPlayStoreInstall(context, it.packageName)
                }

            val firstInstallTimes = applicationInfos.map {
                packageManager.getPackageInfo(it.packageName, 0).firstInstallTime
            }

            val minInstallTime = firstInstallTimes.minOf {
                it
            }

            minInstallTime
        }, 0)
    }

    private fun isSystemPackage(applicationInfo: ApplicationInfo): Boolean {
        return applicationInfo.flags and ApplicationInfo.FLAG_SYSTEM !== 0
    }

    private fun getIsPlayStoreInstall(context: Context, packageName: String): Boolean {
        return executeSafe({
            val validInstallers: List<String> =
                ArrayList(listOf("com.android.vending", "com.google.android.feedback"))

            // The package name of the app that has installed your app

            // The package name of the app that has installed your app
            val installer: String? =
                context.packageManager.getInstallerPackageName(packageName)

            // true if your app has been downloaded from Play Store

            // true if your app has been downloaded from Play Store
            installer != null && validInstallers.contains(installer)
        }, false)
    }
}
