package com.bureau.devicefingerprint.datacollectors

import android.Manifest
import android.content.ContentValues
import android.content.Context
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.provider.MediaStore
import androidx.core.app.ActivityCompat
import com.bureau.devicefingerprint.BureauAPI
import com.bureau.devicefingerprint.tools.executeSafe
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.util.*
import kotlin.jvm.Throws

class FileProvisionsProvider(var context: Context) {

    fun getUniqueIDFromFile(): String {
        return executeSafe({ getUniqueIDFromFileProvider() }, "")
    }

    @Throws(Exception::class)
    private fun getUniqueIDFromFileProvider(): String {
        if (ActivityCompat.checkSelfPermission(
                context,
                Manifest.permission.WRITE_EXTERNAL_STORAGE
            ) == PackageManager.PERMISSION_GRANTED &&
            ActivityCompat.checkSelfPermission(
                    context,
                    Manifest.permission.READ_EXTERNAL_STORAGE
                ) == PackageManager.PERMISSION_GRANTED
        ) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                if (ActivityCompat.checkSelfPermission(
                        context,
                        Manifest.permission.MANAGE_EXTERNAL_STORAGE
                    ) == PackageManager.PERMISSION_GRANTED
                ) {
                    try {
                        val contentResolver = context.contentResolver
                        val file =
                            File(Environment.getExternalStoragePublicDirectory("")?.path + "/" + Environment.DIRECTORY_DOCUMENTS + "/PrismFiles/prism_Id.txt")

                        if (file.exists()) {
                            val i =
                                Uri.fromFile(File(Environment.getExternalStoragePublicDirectory("")?.path + "/" + Environment.DIRECTORY_DOCUMENTS + "/PrismFiles/prism_Id.txt"))

                            val bytes = ByteArray(file.length().toInt())

                            val `in` = contentResolver.openOutputStream(i)
                            `in`.use {
                                it?.write(bytes)
                            }

                            val contents = String(bytes)
                            return executeSafe({ contents }, "")
                        } else {
                            var values = ContentValues()
                            values.put(MediaStore.MediaColumns.DISPLAY_NAME, "prism_Id.txt")
                            values.put(MediaStore.MediaColumns.MIME_TYPE, "text/plain")
                            values.put(MediaStore.MediaColumns.IS_PENDING, 1)
                            values.put(
                                MediaStore.MediaColumns.RELATIVE_PATH,
                                Environment.DIRECTORY_DOCUMENTS + "/PrismFiles"
                            )

                            val mediaUri = contentResolver.insert(
                                MediaStore.Files.getContentUri(MediaStore.VOLUME_EXTERNAL_PRIMARY),
                                values
                            )

                            val out = mediaUri?.let { contentResolver.openOutputStream(it) }
                            // Write your data here
                            val data = UUID.randomUUID().toString()

                            out?.write(data.toByteArray())

                            values = ContentValues()
                            values.put(MediaStore.MediaColumns.IS_PENDING, 0)

                            mediaUri?.let { contentResolver.update(it, values, null, null) }
                            return executeSafe({ data }, "")
                        }
                    } catch (e: Exception) {
                        BureauAPI.handleException(e)
                        return ""
                    }
                } else {
                    return ""
                }
            } else {
                try {
                    val dirName = "PrismFiles"
                    val myDir = File("sdcard", dirName)

                    if (!myDir.exists()) myDir.mkdirs()
                    val id: String
                    val file = File(myDir, "prism_Id.txt")
                    if (file.exists()) {
                        val length = file.length().toInt()

                        val bytes = ByteArray(length)

                        val `in` = FileInputStream(file)
                        `in`.use {
                            it.read(bytes)
                        }

                        val contents = String(bytes)
                        return executeSafe({ contents }, "")
                    } else {
                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                            if (!Environment.isExternalStorageLegacy()) {
                                return ""
                            }
                        }
                        val stream = FileOutputStream(file)
                        stream.use {
                            id = UUID.randomUUID().toString()
                            it.write(id.toByteArray())
                        }
                        return executeSafe({
                            id
                        }, "")
                    }
                } catch (e: Exception) {
                    BureauAPI.handleException(e)
                    return ""
                }
            }
        } else {
            return ""
        }
    }
}
