package com.bureau.devicefingerprint.datacollectors

import android.os.Build
import androidx.core.hardware.fingerprint.FingerprintManagerCompat
import com.bureau.devicefingerprint.tools.executeSafe

interface FingerprintSensorInfoProvider {
    fun getStatus(): FingerprintSensorStatus
}

internal class FingerprintSensorInfoProviderImpl(
    private val fingerprintManager: FingerprintManagerCompat
) : FingerprintSensorInfoProvider {
    override fun getStatus(): FingerprintSensorStatus {
        return executeSafe(
            {
                if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
                    FingerprintSensorStatus.NOT_SUPPORTED
                } else if (!fingerprintManager.isHardwareDetected) {
                    FingerprintSensorStatus.NOT_SUPPORTED
                } else if (!fingerprintManager.hasEnrolledFingerprints()) {
                    FingerprintSensorStatus.SUPPORTED
                } else {
                    FingerprintSensorStatus.ENABLED
                }
            },
            FingerprintSensorStatus.UNKNOWN
        )
    }
}

enum class FingerprintSensorStatus {
    NOT_SUPPORTED,
    SUPPORTED,
    ENABLED,
    UNKNOWN
}
