package com.bureau.devicefingerprint.datacollectors

import android.content.Context
import android.content.pm.PackageManager

class HookDetectionCheckProvider(private val context: Context) {
    init {
        System.loadLibrary("native-lib")
    }
    private external fun detectFrida(): Boolean

    fun isHookingDetected() = detectFrida() || hookDetected()

    /**
     * Detects if there is any suspicious installed application.
     *
     * @return `true` if some bad application is installed, `false` otherwise.
     */
    private fun hookDetected(): Boolean {
        val packageManager = context.packageManager
        val applicationInfoList =
            packageManager.getInstalledApplications(PackageManager.GET_META_DATA)
        val dangerousPackages = listOf(
            "de.robv.android.xposed.installer",
            "com.saurik.substrate",
            "de.robv.android.xposed"
        )
        for (applicationInfo in applicationInfoList) {
            if (dangerousPackages.contains(applicationInfo.packageName)) {
                return true
            }
        }
        return advancedHookDetection()
    }

    private fun advancedHookDetection(): Boolean {
        try {
            throw Exception()
        } catch (e: Exception) {
            var zygoteInitCallCount = 0
            for (stackTraceElement in e.stackTrace) {
                if (stackTraceElement.className == "com.android.internal.os.ZygoteInit") {
                    zygoteInitCallCount++
                    if (zygoteInitCallCount == 2) {
                        return true
                    }
                }
                if (stackTraceElement.className == "com.saurik.substrate.MS$2" && stackTraceElement.methodName == "invoked") {
                    return true
                }
                if (stackTraceElement.className == "de.robv.android.xposed.XposedBridge" && stackTraceElement.methodName == "main") {
                    return true
                }
                if (stackTraceElement.className == "de.robv.android.xposed.XposedBridge" && stackTraceElement.methodName == "handleHookedMethod") {
                    return true
                }
            }
        }
        return false
    }
}
