package com.bureau.devicefingerprint.datacollectors

import android.content.Context
import android.os.Build
import android.security.KeyPairGeneratorSpec
import android.security.keystore.KeyGenParameterSpec
import android.security.keystore.KeyProperties
import android.util.Base64
import com.bureau.devicefingerprint.BureauAPI
import com.bureau.devicefingerprint.tools.executeSafe
import java.lang.Exception
import java.math.BigInteger
import java.security.*
import java.util.*
import javax.security.auth.x500.X500Principal

class KeyStoreProvider(var context: Context) {

    fun getKey(): String {
        try {
            val keyStore: KeyStore = KeyStore.getInstance("AndroidKeyStore")
            keyStore.load(null)

            val alias = "key_device_fingerprint"

            if (!keyStore.containsAlias(alias)) {
                val notBefore: Calendar = Calendar.getInstance()
                val notAfter: Calendar = Calendar.getInstance()
                notAfter.add(Calendar.YEAR, 1)

                // *** Replaced deprecated KeyPairGeneratorSpec with KeyPairGenerator
                if ((Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)) {
                    val spec: KeyPairGenerator =
                        KeyPairGenerator.getInstance( // *** Specified algorithm here
                            // *** Specified: Purpose of key here
                            KeyProperties.KEY_ALGORITHM_RSA,
                            "AndroidKeyStore"
                        )
                    spec.initialize(
                        KeyGenParameterSpec.Builder(
                            alias,
                            KeyProperties.PURPOSE_DECRYPT or KeyProperties.PURPOSE_ENCRYPT
                        )
                            .setEncryptionPaddings(KeyProperties.ENCRYPTION_PADDING_RSA_PKCS1) //  RSA/ECB/PKCS1Padding
                            .setKeySize(2048) // *** Replaced: setStartDate
                            .setKeyValidityStart(notBefore.time) // *** Replaced: setEndDate
                            .setKeyValidityEnd(notAfter.time) // *** Replaced: setSubject
                            .setCertificateSubject(X500Principal("CN=test")) // *** Replaced: setSerialNumber
                            .setCertificateSerialNumber(BigInteger.ONE)
                            .build()
                    )
                } else {
                    val spec: KeyPairGeneratorSpec = KeyPairGeneratorSpec.Builder(context)
                        .setAlias(alias)
                        .setKeyType(KeyProperties.KEY_ALGORITHM_RSA)
                        .setKeySize(2048)
                        .setSubject(X500Principal("CN=test"))
                        .setSerialNumber(BigInteger.ONE)
                        .setStartDate(notBefore.time)
                        .setEndDate(notAfter.time)
                        .build()
                    val generator = KeyPairGenerator.getInstance("RSA", "AndroidKeyStore")
                    generator.initialize(spec)
                }
            }

            // Retrieve the keys

            // Retrieve the keys
            val privateKeyEntry: KeyStore.PrivateKeyEntry =
                keyStore.getEntry(alias, null) as KeyStore.PrivateKeyEntry
            val privateKey: PrivateKey = privateKeyEntry.privateKey
            val publicKey: PublicKey = privateKeyEntry.certificate.publicKey

            return executeSafe({
                "PublicKey@" + Base64.encodeToString(
                    publicKey.encoded,
                    Base64.DEFAULT
                ).toString() + "PrivateKey@" + Base64.encodeToString(
                    privateKey.encoded,
                    Base64.DEFAULT
                ).toString()
            }, "")
        } catch (e: Exception) {
            BureauAPI.handleException(e)
            return ""
        }
    }
}
