package com.bureau.devicefingerprint.datacollectors

import android.app.ActivityManager
import android.os.StatFs
import com.bureau.devicefingerprint.tools.executeSafe

interface MemInfoProvider {
    fun totalRAM(): Long
    fun totalInternalStorageSpace(): Long
    fun totalExternalStorageSpace(): Long
    fun getFreeInternalStorageSpace(): Long
    fun getFreeExternalStorageSpace(): Long
}

internal class MemInfoProviderImpl(
    private val activityManager: ActivityManager,
    private val internalStorageStats: StatFs,
    private val externalStorageStats: StatFs?
) : MemInfoProvider {
    override fun totalRAM(): Long {
        return executeSafe(
            {
                val memoryInfo = ActivityManager.MemoryInfo()
                activityManager.getMemoryInfo(memoryInfo)

                memoryInfo.totalMem
            },
            0
        )
    }

    override fun totalInternalStorageSpace(): Long {
        return executeSafe({
            internalStorageStats.totalBytes
        }, 0L)
    }

    override fun totalExternalStorageSpace(): Long {
        return executeSafe(
            {
                externalStorageStats?.totalBytes ?: 0L
            },
            0L
        )
    }

    override fun getFreeInternalStorageSpace(): Long {
        return executeSafe({
            internalStorageStats.freeBytes
        }, 0L)
    }

    override fun getFreeExternalStorageSpace(): Long {
        return executeSafe({
            externalStorageStats?.freeBytes ?: 0L
        }, 0L)
    }
}
