package com.bureau.devicefingerprint.datacollectors

import android.content.Context
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.os.Build
import com.bureau.devicefingerprint.tools.SystemPropertiesInvoker
import com.bureau.devicefingerprint.tools.executeSafe
import java.net.Inet4Address
import java.net.Inet6Address
import java.net.InetAddress
import java.net.NetworkInterface
import java.util.*

class NetworkInfoProvider(var context: Context) {
    fun getIpv6Address(): String? {
        return executeSafe({
            val en = NetworkInterface.getNetworkInterfaces()
            while (en.hasMoreElements()) {
                val intf = en.nextElement()
                val enumIpAddr: Enumeration<InetAddress> = intf.inetAddresses
                while (enumIpAddr.hasMoreElements()) {
                    val inetAddress: InetAddress = enumIpAddr.nextElement()
                    if (!inetAddress.isLoopbackAddress && inetAddress is Inet6Address) {
                        return@executeSafe inetAddress.getHostAddress()
                    }
                }
            }

            ""
        }, "")
    }

    fun getIpv4Address(): String? {
        return executeSafe({
            val en = NetworkInterface.getNetworkInterfaces()
            while (en.hasMoreElements()) {
                val intf = en.nextElement()
                val enumIpAddr: Enumeration<InetAddress> = intf.inetAddresses
                while (enumIpAddr.hasMoreElements()) {
                    val inetAddress: InetAddress = enumIpAddr.nextElement()
                    if (!inetAddress.isLoopbackAddress && inetAddress is Inet4Address) {
                        return@executeSafe inetAddress.getHostAddress()
                    }
                }
            }

            ""
        }, "")
    }

    fun isConnectedToVPN(): Boolean {
        val connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        return executeSafe({
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                val activeNetwork: Network? = connectivityManager.activeNetwork
                val caps: NetworkCapabilities? =
                    connectivityManager.getNetworkCapabilities(activeNetwork)
                return@executeSafe caps?.hasTransport(NetworkCapabilities.TRANSPORT_VPN) == true
            } else {
                val networks: Array<Network> = connectivityManager.allNetworks

                for (i in networks.indices) {
                    val caps: NetworkCapabilities? = connectivityManager.getNetworkCapabilities(networks[i])
                    return@executeSafe caps?.hasTransport(NetworkCapabilities.TRANSPORT_VPN) == true
                }
                return@executeSafe false
            }
        }, false)
    }

    fun getWifiMACAddress(): String {
        return executeSafe({
            SystemPropertiesInvoker.getMACAddress("wlan0") ?: ""
        }, "")
    }

    fun getEthernetMACAddress(): String {
        return executeSafe({
            SystemPropertiesInvoker.getMACAddress("eth0") ?: ""
        }, "")
    }

    fun getIsWifiConnected(): Boolean {
        val connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        return executeSafe({
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                val networkCapabilities = connectivityManager.getNetworkCapabilities(connectivityManager.activeNetwork)
                return@executeSafe networkCapabilities?.hasTransport(NetworkCapabilities.TRANSPORT_WIFI) == true
            } else {
                val activeNetworkInfo = connectivityManager.activeNetworkInfo
                return@executeSafe activeNetworkInfo != null && activeNetworkInfo.type == ConnectivityManager.TYPE_WIFI
            }
        }, false)
    }

    fun getIsCellularConnected(): Boolean {
        val connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        return executeSafe({
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                val networkCapabilities = connectivityManager.getNetworkCapabilities(connectivityManager.activeNetwork)
                return@executeSafe networkCapabilities?.hasTransport(NetworkCapabilities.TRANSPORT_CELLULAR) == true
            } else {
                val activeNetworkInfo = connectivityManager.activeNetworkInfo
                return@executeSafe activeNetworkInfo != null && activeNetworkInfo.type == ConnectivityManager.TYPE_MOBILE
            }
        }, false)
    }
}
