package com.bureau.devicefingerprint.datacollectors

import android.os.Build
import android.os.SystemClock
import com.bureau.devicefingerprint.tools.executeSafe

interface OsBuildInfoProvider {
    fun modelName(): String
    fun manufacturerName(): String
    fun androidVersion(): String
    fun sdkVersion(): String
    fun kernelVersion(): String
    fun fingerprint(): String
    fun getElapsedTime(): Long
    fun device(): String
    fun board(): String
    fun product(): String
    fun host(): String
    fun brand(): String
    fun getBuildId(): String
    fun getBuildTime(): String
}

internal class OsBuildInfoProviderImpl :
    OsBuildInfoProvider {
    override fun modelName(): String {
        return executeSafe({ Build.MODEL }, "")
    }

    override fun manufacturerName(): String {
        return executeSafe({ Build.MANUFACTURER }, "")
    }

    override fun androidVersion(): String {
        return executeSafe({ Build.VERSION.RELEASE }, "")
    }

    override fun sdkVersion(): String {
        return executeSafe({ Build.VERSION.SDK_INT.toString() }, "")
    }

    override fun device(): String {
        return executeSafe({ Build.DEVICE.toString() }, "")
    }

    override fun host(): String {
        return executeSafe({ Build.HOST.toString() }, "")
    }

    override fun brand(): String {
        return executeSafe({ Build.BRAND.toString() }, "")
    }

    override fun getBuildId(): String {
        return executeSafe({ Build.ID.toString() }, "")
    }

    override fun getBuildTime(): String {
        return executeSafe({ Build.TIME.toString() }, "")
    }

    override fun board(): String {
        return executeSafe({ Build.BOARD.toString() }, "")
    }

    override fun product(): String {
        return executeSafe({ Build.PRODUCT.toString() }, "")
    }

    override fun kernelVersion(): String {
        return executeSafe({ System.getProperty("os.version") ?: "" }, "")
    }

    override fun fingerprint(): String {
        return executeSafe({ Build.FINGERPRINT }, "")
    }

    override fun getElapsedTime(): Long {
        return executeSafe({ SystemClock.elapsedRealtime() }, 0)
    }
}
