package com.bureau.devicefingerprint.datacollectors

import android.annotation.SuppressLint
import android.content.Context
import android.content.pm.PackageManager
import com.bureau.devicefingerprint.tools.executeSafe

interface PackageManagerDataSource {
    fun getApplicationsList(): List<PackageInfo>
    fun getSystemApplicationsList(): List<PackageInfo>
    fun getSystemApplicationListInString(): String
    fun getApplicationListInString(): String
    fun getPackageVersion(): String
}

data class PackageInfo(
    val packageName: String
) {
    override fun toString(): String {
        return packageName
    }
}

internal class PackageManagerDataSourceImpl(
    private val packageManager: PackageManager,
    private val context: Context
) : PackageManagerDataSource {
    @SuppressLint("QueryPermissionsNeeded")
    override fun getApplicationsList() = executeSafe(
        /*{
            packageManager
                .getInstalledApplications(PackageManager.GET_META_DATA)
                .map {
                    PackageInfo(
                        it.packageName
                    )
                }
        }*/
        {
            emptyList<PackageInfo>()
        },
        emptyList()
    )

    @SuppressLint("QueryPermissionsNeeded")
    override fun getSystemApplicationsList() = executeSafe(
        /*{
            packageManager
                .getInstalledApplications(PackageManager.GET_META_DATA)
                .filter {
                    it.sourceDir.contains("/system/")
                }
                .map {
                    PackageInfo(
                        it.packageName
                    )
                }
        }*/
        {
            emptyList<PackageInfo>()
        },
        emptyList()
    )

    override fun getApplicationListInString(): String {
        val sb = StringBuilder()

        getApplicationsList()
            .sortedBy { it.packageName }
            .forEach { sb.append(it.packageName) }

        return sb.toString()
    }

    override fun getPackageVersion(): String {
        val packageInfo: android.content.pm.PackageInfo? =
            packageManager.getPackageInfo(context.packageName, 0)
        return executeSafe({
            packageInfo?.versionName ?: ""
        }, "")
    }

    override fun getSystemApplicationListInString(): String {
        val sb = StringBuilder()

        getSystemApplicationsList()
            .sortedBy { it.packageName }
            .forEach { sb.append(it.packageName) }

        return sb.toString()
    }
}
