package com.bureau.devicefingerprint.datacollectors

import android.content.Context
import android.content.res.Configuration
import android.hardware.display.DisplayManager
import android.util.DisplayMetrics
import android.view.WindowManager
import com.bureau.devicefingerprint.tools.executeSafe

class ScreenInfoProvider(var context: Context) {
    private var displayManager: DisplayManager = context.getSystemService(Context.DISPLAY_SERVICE) as DisplayManager

    fun getScreenCount(): Int {
        return executeSafe({
            displayManager.displays.size
        }, -1)
    }

    fun getDrawableDensity(): Int {
        return executeSafe({
            context.resources.displayMetrics.densityDpi
        }, -1)
    }

    fun getLayoutSize(): String {
        return executeSafe({
            resolveScreenSize()
        }, "")
    }

    fun getLayoutWidth(): Int {
        return executeSafe({
            val displayMetrics = DisplayMetrics()
            val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
            windowManager.defaultDisplay.getRealMetrics(displayMetrics)
            displayMetrics.widthPixels
        }, -1)
    }

    fun getLayoutHeight(): Int {
        return executeSafe({
            val displayMetrics = DisplayMetrics()
            val windowManager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
            windowManager.defaultDisplay.getRealMetrics(displayMetrics)
            displayMetrics.heightPixels
        }, -1)
    }

    private fun resolveScreenSize(): String {
        val screenSize: Int = context.resources.configuration.screenLayout and
            Configuration.SCREENLAYOUT_SIZE_MASK

        val toastMsg: String = when (screenSize) {
            Configuration.SCREENLAYOUT_SIZE_LARGE -> "Large screen"
            Configuration.SCREENLAYOUT_SIZE_NORMAL -> "Normal screen"
            Configuration.SCREENLAYOUT_SIZE_SMALL -> "Small screen"
            else -> "Screen size is neither large, normal or small"
        }
        return toastMsg
    }
}
