package com.bureau.devicefingerprint.datacollectors

import android.content.Context
import android.telephony.CellInfoGsm
import android.telephony.CellInfoLte
import android.telephony.CellInfoWcdma
import android.telephony.TelephonyManager
import android.text.TextUtils
import android.util.Log
import androidx.appcompat.app.AppCompatActivity
import com.bureau.devicefingerprint.models.devicedataholder.TelecomBaseStationInfo
import com.bureau.devicefingerprint.tools.executeSafe

class TelecomBaseStationInfoProvider(private val context: Context) {
    fun getTelecomBaseStationInfo(): TelecomBaseStationInfo {
        return executeSafe({
            var mcc = ""
            var mnc = ""
            var cid = ""
            var lac = ""
            var type = ""
            val tel =
                context.getSystemService(AppCompatActivity.TELEPHONY_SERVICE) as TelephonyManager
            val networkOperator = tel.simOperator
            if (!TextUtils.isEmpty(networkOperator)) {
                mcc = networkOperator.substring(0, 3)
                mnc = networkOperator.substring(3)
                Log.d("balatag", "onResume: $mcc $mnc")
            }
            val cellInfoList = tel.getAllCellInfo()
            if (cellInfoList != null && cellInfoList.isNotEmpty()) {
                when (val mainBSCellInfo = cellInfoList.get(0)) {
                    is CellInfoWcdma -> {
                        // 联通3G
                        val cellIdentityWcdma = mainBSCellInfo.cellIdentity
                        type = "WCDMA"
                        cid = cellIdentityWcdma.cid.toString()
                        lac = cellIdentityWcdma.lac.toString()
                    }

                    is CellInfoLte -> {
                        // 4G
                        val cellIdentityLte = mainBSCellInfo.cellIdentity
                        type = "LTE"
                        cid = cellIdentityLte.ci.toString()
                        lac = cellIdentityLte.tac.toString()
                    }

                    is CellInfoGsm -> {
                        // 2G
                        val cellIdentityGsm = mainBSCellInfo.cellIdentity
                        type = "GSM"
                        cid = cellIdentityGsm.cid.toString()
                        lac = cellIdentityGsm.lac.toString()
                    }

                    else -> {
                        // 电信2/3G
                        Log.e(
                            "balatag",
                            "CDMA CellInfo................................................"
                        )
                    }
                }
            }

            TelecomBaseStationInfo(type, mcc, mnc, lac, cid)
        }, TelecomBaseStationInfo())
    }
}
