package com.bureau.devicefingerprint.tools

import com.bureau.devicefingerprint.BureauAPI
import org.json.JSONObject
import java.io.BufferedReader
import java.io.InputStreamReader
import java.lang.Exception
import java.net.NetworkInterface
import java.util.*

object SystemPropertiesInvoker {
    fun getSystemProperty(propertyName: String): String {
        var propertyValue = "[UNKNOWN]"

        try {
            val getPropProcess: Process = Runtime.getRuntime().exec("getprop $propertyName")

            val osRes =
                BufferedReader(InputStreamReader(getPropProcess.inputStream))

            propertyValue = osRes.readLine()

            osRes.close()
        } catch (e: Exception) {
            // Do nothing - can't get property value
            BureauAPI.handleException(e)
        }

        return propertyValue
    }

    fun getSystemProperties(): List<String> {
        var systemProperties = listOf<String>()

        try {
            val getPropProcess: Process = Runtime.getRuntime().exec("getprop")

            val osRes =
                BufferedReader(InputStreamReader(getPropProcess.inputStream))

            systemProperties = osRes.readLines()

            osRes.close()
        } catch (e: Exception) {
            // Do nothing - can't get property value
            BureauAPI.handleException(e)
        }

        return systemProperties
    }

    fun getMACAddress(interfaceName: String?): String {
        try {
            val interfaces: List<NetworkInterface> =
                Collections.list(NetworkInterface.getNetworkInterfaces())
            for (intf in interfaces) {
                if (interfaceName != null) {
                    if (!intf.name.equals(interfaceName, true)) continue
                }
                val mac: ByteArray = intf.hardwareAddress ?: return ""
                val buf = StringBuilder()
                for (aMac in mac) buf.append(String.format("%02X:", aMac))
                if (buf.isNotEmpty()) buf.deleteCharAt(buf.length - 1)
                return buf.toString()
            }
        } catch (ignored: Exception) {
            BureauAPI.handleException(ignored)
        } // for now eat exceptions
        return ""
        /*try {
            // this is so Linux hack
            return loadFileAsString("/sys/class/net/" +interfaceName + "/address").toUpperCase().trim();
        } catch (IOException ex) {
            return null;
        }*/
    }

    fun jsonToMap(string: String?): HashMap<String, String> {
        val map = HashMap<String, String>()
        val jObject = JSONObject(string ?: "")
        val keys: Iterator<*> = jObject.keys()

        while (keys.hasNext()) {
            val key = keys.next() as String
            val value: String = jObject.getString(key)
            map[key] = value
        }
        return map
    }
}
